import tkinter as tk
from tkinter import ttk
import pygame
import subprocess
import os
import sys
from pathlib import Path
from threading import Thread


parent_directory = Path(__file__).resolve().parent.parent
kernel_directory = parent_directory / "Kernel"
sys.path.append(str(kernel_directory))
print(f"Kernel directory: {kernel_directory}")

from screener import StockList, main as screener_main
import pickle

from broker import run_backtest

import client_starter 






def main():
    root = tk.Tk()
    root.withdraw()

    window4 = None

   
    window4 = None

    def create_window1():
        window1 = tk.Toplevel(root)
        window1.title("Window 1")
        window1.minsize(width=300,height=300)

        choices = ["Choice 1", "Choice 2", "Choice 3"]
        choice_var = tk.StringVar()

        def on_choice_selected():
            choice = choice_var.get()
            label.config(text=f"{choice} is active")

        for choice in choices:
            b = tk.Radiobutton(window1, text=choice, variable=choice_var, value=choice, command=on_choice_selected, indicatoron=0, width=15, bg="green" if choice_var.get() == choice else "#FFA07A")
            b.pack(fill="x")

        label = tk.Label(window1, text="No choice selected")
        label.pack()

    def create_window2():
        window2 = tk.Toplevel(root)
        window2.title("Window 2")
        window2.minsize(width=300,height=300)

        stocks = {"AAPL": ["2020-01-01", "2021-01-01", "2022-01-01"],
                "GOOGL": ["2019-01-01", "2020-01-01", "2021-01-01"],
                "TSLA": ["2018-01-01", "2019-01-01", "2020-01-01"]}

        stock_var = tk.StringVar()
        date_var = tk.StringVar()
        date_var.set("Select Date")

        def on_stock_selected(*_):
            stock = stock_var.get()
            date_menu.delete(0, "end")
            for date in stocks[stock]:
                date_menu.insert("end", date)
            label.config(text=f"{stock} is active")
            use_data_button.config(state="disabled")

        def on_date_selected(event):
            if date_menu.curselection():
                index = date_menu.curselection()[0]
                date = date_menu.get(index)
                date_var.set(date)
                label.config(text=f"Date {date} is active")
                use_data_button.config(state="normal")
            else:
                use_data_button.config(state="disabled")

        stock_menu = tk.OptionMenu(window2, stock_var, *stocks.keys(), command=on_stock_selected)
        stock_menu.pack()

        date_menu = tk.Listbox(window2, selectmode="SINGLE", listvariable=date_var, exportselection=False)
        date_menu.pack()
        date_menu.bind("<<ListboxSelect>>", on_date_selected)

        label = tk.Label(window2, text="No stock/date selected")
        label.pack()

        def use_data():
            selected_stock = stock_var.get()
            selected_date = date_var.get()
            new_window = tk.Toplevel(root)
            new_window.title(f"Data for {selected_stock} on {selected_date}")
            tk.Label(new_window, text=f"Using data for {selected_stock} on {selected_date}").pack()

        use_data_button = tk.Button(window2, text="Use this historical price action", state="disabled", command=use_data)
        use_data_button.pack()

    def create_window4():
        nonlocal window4
        
        window4 = tk.Toplevel(root)
        window4.title("Window 4")
        window4.minsize(width=300,height=300)



        def start_screener():
            subprocess.Popen(["python", str(kernel_directory / "screener.py")])

        start_screener_button = tk.Button(window4, text="Start Screener", command=start_screener)
        start_screener_button.pack()

    def create_window5():
        window5 = tk.Toplevel(root)
        window5.title("Chosen Stocks")
        window5.minsize(width=300, height=300)

        stocks_listbox = tk.Listbox(window5)
        stocks_listbox.pack(fill="both", expand=True)

        def refresh_stocks_list():
            stocks_listbox.delete(0, "end")
            if not os.path.exists('selected_stocks.pkl'):  # Check if the file exists
                with open('selected_stocks.pkl', 'wb') as f:  # If not, create an empty file
                    pickle.dump([], f)
            with open('selected_stocks.pkl', 'rb') as f:
                stocks = pickle.load(f)
            for stock in stocks:
                stocks_listbox.insert("end", stock)

        refresh_button = tk.Button(window5, text="Refresh", command=refresh_stocks_list)
        refresh_button.pack()

        # Call refresh_stocks_list once when the window is created
        refresh_stocks_list()


    def create_window3():
        window3 = tk.Toplevel(root)
        window3.title("Window 3")

        colors = ["#FF5733", "#9B59B6", "#3498DB", "#1ABC9C", "#28B463", "#F1C40F", "#E74C3C", "#D35400", "#2C3E50"]
        buttons = []

        for i in range(27):
            b = tk.Button(window3, text=f"Button {i+1}", bg=colors[i % len(colors)], width=20, height=2)
            buttons.append(b)
            b.grid(row=i // 9, column=i % 9)

        buttons[0].config(text="Options", command=create_window1)
        buttons[1].config(text="Stocks and historical dates", command=create_window2)
        buttons[26].config(text="Close the application completely", command=root.destroy)

    def create_window6():
        window6 = tk.Toplevel(root)
        window6.title("Backtesting Results")
        window6.minsize(width=300, height=300)

        def run_backtest_on_selected_stocks():
            if not os.path.exists('selected_stocks.pkl'):  # Check if the file exists
                with open('selected_stocks.pkl', 'wb') as f:  # If not, create an empty file
                    pickle.dump([], f)
            with open('selected_stocks.pkl', 'rb') as f:
                stocks = pickle.load(f)
            run_backtest(stocks)

        run_backtest_button = tk.Button(window6, text="Run Backtest", command=run_backtest_on_selected_stocks)
        run_backtest_button.pack()

    def create_window7():
        window7 = tk.Toplevel(root)
        window7.title("Trading Client")
        window7.minsize(width=500, height=500)

        # Create an instance of the TradingClient class
        def create_window7():
            client_starter.main()



    create_window1()
    create_window2()
    create_window3()
    create_window4()
    create_window5()
    create_window6()
    create_window7()
    root.mainloop()

if __name__ == "__main__":
    main()
