import os
import glob
import pandas as pd
import ta
import numpy as np
import scipy.signal
import mplfinance as mpf
import yfinance as yf
import pandas_ta as pta

def true_range(high, low, close, prev_close):
    if pd.isna(prev_close):
        return high - low
    else:
        return max(high - low, abs(high - prev_close), abs(low - prev_close))

def wma(arr: np.ndarray, weights: np.ndarray) -> np.ndarray:
    weighted_arr = np.multiply(arr, weights)
    return np.sum(weighted_arr, axis=-1) / np.sum(weights)

def calculate_indicators(df):
    #ošetření množství dat
    if len(df) < 10:
        return df

    # Simple Moving Average (SMA)
    df['SMA_10'] = df['close'].rolling(window=10).mean()

    # Exponential Moving Average (EMA)
    df['EMA_10'] = df['close'].ewm(span=10).mean()
    print(df.shape)
    # Weighted Moving Average (WMA)
    weights = np.arange(1, 11)
    df['WMA_10'] = df['close'].rolling(window=10).apply(lambda x: wma(x, weights), raw=True)
    print(df.shape) 
    # Moving Average Convergence Divergence (MACD)
    df['MACD'] = ta.trend.MACD(df['close']).macd()

    # Relative Strength Index (RSI)
    df['RSI'] = ta.momentum.RSIIndicator(df['close']).rsi()

    # Stochastic Oscillator
    stoch = ta.momentum.StochasticOscillator(df['high'], df['low'], df['close'])
    df['%K'] = stoch.stoch()
    df['%D'] = stoch.stoch_signal()

    # Bollinger Bands
    bollinger = ta.volatility.BollingerBands(df['close'])
    df['BB_upper'] = bollinger.bollinger_hband()
    df['BB_lower'] = bollinger.bollinger_lband()

    # Commodity Channel Index (CCI)
    df['CCI'] = ta.trend.cci(df['high'], df['low'], df['close'])
    print(df.shape)
    # On Balance Volume (OBV)
    df['OBV'] = ta.volume.on_balance_volume(df['close'], df['volume'])

    # Fibonacci Retracement Levels
    min_close = df['close'].min()
    max_close = df['close'].max()
    fib_levels = [0, 0.236, 0.382, 0.5, 0.618, 0.786, 1]
    fib_retracement = [(min_close + level * (max_close - min_close)) for level in fib_levels]

    # Add Fibonacci Retracement Levels columns
    for i, level in enumerate(fib_levels):
        df[f'Fibonacci_{level}'] = fib_retracement[i]

    print(df.shape)
    # Parabolic SAR
    df['Parabolic_SAR'] = ta.trend.PSARIndicator(df['high'], df['low'], df['close']).psar()

    # True Range
    df['True_Range'] = df.apply(lambda x: true_range(x['high'], x['low'], x['close'], df['close'].shift(1).loc[x.name]), axis=1)
 


    # Detrended Price Oscillator (DPO)
    df['DPO'] = pta.dpo(df['close'])

 
    adx_df = pta.adx(df['high'], df['low'], df['close'])
    df = pd.concat([df, adx_df], axis=1)


    # Standard Deviation
    df['Standard_Deviation'] = df['close'].rolling(window=10).std()

    # Triangular Moving Average (TMA)
    df['TMA'] = df['close'].rolling(window=10, center=True, win_type='triang').mean()

    # Keltner Channel
    keltner = ta.volatility.KeltnerChannel(df['high'], df['low'], df['close'])
    df['KC_upper'] = keltner.keltner_channel_hband()
    df['KC_lower'] = keltner.keltner_channel_lband()

    return df

def process_stock_files(input_folder, output_folder):
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    file_paths = glob.glob(f"{input_folder}/*_5min_normalized.csv")

    for file_path in file_paths:
        stock_name = os.path.basename(file_path).split("_")[0]

        df = pd.read_csv(file_path)
        df['timestamp'] = pd.to_datetime(df['timestamp'])
        df.set_index('timestamp', inplace=True)

        df_indicators = calculate_indicators(df)

        output_path = os.path.join(output_folder, f"{stock_name}_5min_normalized_indicators.csv")
        df_indicators.to_csv(output_path)

input_folder = 'Stockdata_normalized'
output_folder = 'stockDataIndicators'
process_stock_files(input_folder, output_folder)
