import datetime
import numpy as np
import pandas as pd
from alpha_vantage.timeseries import TimeSeries
from ib_insync import IB, Stock, util
import requests
from ib_insync import MarketOrder
from collections import deque
import talib









# Replace 'YOUR_API_KEY' with your actual AlphaVantage API key
alpha_vantage_api_key = 'JACJKLH87O6AUMBM'

def run_backtest_for_stock(symbol):
    # Connect to Interactive Brokers
    ib = IB()
    ib.connect('127.0.0.1', 7497, clientId=1)

    # Add these lines after defining the get_position_size function
    cumulative_pnl = 0
    num_trades = 0
    winning_trades = 0
    losing_trades = 0
    max_winning_trade = 0
    max_losing_trade = 0
    equity_curve = deque(maxlen=1000)
    drawdown = 0

    # Define the stock symbol and the exchange
    symbol = symbol
    exchange = 'SMART'


    # Define the historical data request duration and bar size
    duration = '1 D'
    bar_size = '1 min'

    # Define the risk management and position sizing parameters
    risk_per_trade = 0.01
    account_equity = 100000  # Adjust this to your account size

        # Create a contract for the stock and qualify it
    contract = Stock(symbol, exchange=exchange, currency='USD')
    qualified_contract = ib.qualifyContracts(contract)[0]


    # Request historical data
    ts = TimeSeries(key=alpha_vantage_api_key, output_format='pandas')
    bars, meta_data = ts.get_intraday(symbol='AAPL', interval='1min', outputsize='full')
    bars = bars.rename(columns={'1. open': 'open', '2. high': 'high', '3. low': 'low', '4. close': 'close', '5. volume': 'volume'})

    bars['SMA10'] = talib.SMA(bars['close'], timeperiod=10)
    bars['SMA20'] = talib.SMA(bars['close'], timeperiod=20)
    bars['EMA10'] = talib.EMA(bars['close'], timeperiod=10)
    bars['EMA20'] = talib.EMA(bars['close'], timeperiod=20)

    entry_condition = lambda row, prev_row: (
        row['high'] > prev_row['high'] and
        row['EMA10'] > row['EMA20'] and
        row['SMA10'] > row['SMA20']
    )

    exit_condition = lambda row, prev_row: (
        row['low'] < prev_row['low']
    )

    def get_position_size(risk_per_trade, account_equity, stop_loss):
        risk_amount = account_equity * risk_per_trade
        position_size = int(risk_amount / stop_loss)
        return position_size



    # Iterate through the bars to find trading opportunities
    in_position = False
    entry_price = None
    stop_loss = None

    for idx, (index, row) in enumerate(bars.iterrows()):
        if idx < 20:  # Skip the first 20 rows to ensure we have enough data for our indicators
            continue

        prev_index = bars.index.get_loc(index) - 1
        previous_row = bars.iloc[prev_index]
        price_volume = row['close'] * row['volume']

        if not in_position and price_volume > 100000 and entry_condition(row, previous_row):
            # Entry condition met
            entry_price = row['high']
            stop_loss = previous_row['low']
            position_size = get_position_size(risk_per_trade, account_equity, stop_loss)

            # Buy the stock
            order = MarketOrder('BUY', position_size)
            trade = ib.placeOrder(qualified_contract, order)
            print(f"Entering long position at {entry_price} with position size {position_size}")

            in_position = True

        elif in_position and exit_condition(row, previous_row):
            # Exit condition met
            exit_price = previous_row['low']
            order = MarketOrder('SELL', position_size)
            trade = ib.placeOrder(qualified_contract, order)

            pnl = (exit_price - entry_price) * position_size
            cumulative_pnl += pnl
            num_trades += 1

            if pnl > 0:
                winning_trades += 1
                max_winning_trade = max(max_winning_trade, pnl)
            else:
                losing_trades += 1
                max_losing_trade = min(max_losing_trade, pnl)

            drawdown = min(drawdown, cumulative_pnl)

        in_position = False
        entry_price = None
        stop_loss = None

    if num_trades > 0:
        winrate = (winning_trades / num_trades) * 100
    else:
        winrate = 0

    print("Final Results:")
    print(f"Cumulative PnL: {cumulative_pnl}")
    print(f"Number of Trades: {num_trades}")
    print(f"Drawdown: {drawdown}")
    print(f"Winrate: {winrate:.2f}%")
    print(f"Winning Trades: {winning_trades}")
    print(f"Losing Trades: {losing_trades}")
    print(f"Max Winning Trade: {max_winning_trade}")
    print(f"Max Losing Trade: {max_losing_trade}")
    # Disconnect from Interactive Brokers
    ib.disconnect()

def run_backtest(stocks):
    for stock in stocks:
        run_backtest_for_stock(stock)

if __name__ == "__main__":
    stocks = ['AAPL', 'GOOGL', 'MSFT']  # Add more stock symbols as desired
    run_backtest(stocks)

