import os
import re
import pandas as pd

def rename_columns(filename):
    # Read the CSV file with header=None
    stock_data = pd.read_csv(filename, header=None)

    try:
        # Assign new column names to the DataFrame
        num_columns = len(stock_data.columns)
        if num_columns == 6:
            stock_data.columns = [
                'timestamp',
                'open',
                'high',
                'low',
                'close',
                'volume'
            ]
        else:
            # handle the case where the number of columns is different
            pass

        # Drop the first row containing the old column names
        stock_data = stock_data.iloc[1:]

        # Convert the 'timestamp' column to datetime format
        stock_data['timestamp'] = pd.to_datetime(stock_data['timestamp'])

        # Sort the data by 'timestamp' in ascending order
        stock_data = stock_data.sort_values(by='timestamp', ascending=True)

        return stock_data
    except KeyError:
        print(f"stock {filename} ignored")    
        return None

def main():
    # Create the "Stockdata_normalized" folder if it doesn't exist
    if not os.path.exists('Stockdata_normalized'):
        os.makedirs('Stockdata_normalized')

    for filename in os.listdir():
        if filename.endswith('_5min.csv'):
            stock_name = filename.split('_')[0]
            modified_data = rename_columns(filename)
            if modified_data is not None:
                output_filename = f"Stockdata_normalized/{stock_name}_5min_normalized.csv"
                modified_data.to_csv(output_filename, index=False)

if __name__ == '__main__':
    main()
