import requests
import pandas as pd
from ib_insync import IB, Stock, Order
import time

ALPHAVANTAGE_API_KEY = "JACJKLH87O6AUMBM"
INTERVAL = "5sec"
STOCKS_TO_TRADE = 5

def get_most_traded_stocks():
    url = f"https://www.alphavantage.co/query?function=SECTOR&apikey={ALPHAVANTAGE_API_KEY}"
    response = requests.get(url).json()
    top_stocks = []

    # Get most active stocks
    most_active = response["Rank A: Real-Time Performance"]
    for stock_symbol, performance in most_active.items():
        top_stocks.append(stock_symbol)

    return top_stocks[:STOCKS_TO_TRADE]

def get_stock_data(symbol):
    url = f"https://www.alphavantage.co/query?function=TIME_SERIES_INTRADAY_EXTENDED&symbol={symbol}&interval={INTERVAL}&apikey={ALPHAVANTAGE_API_KEY}"
    response = requests.get(url).json()

    if "Time Series" not in response:
        return None

    stock_data = response["Time Series"]
    df = pd.DataFrame(stock_data).transpose()
    df.index = pd.to_datetime(df.index)

    return df

def higher_highs_strategy(df):
    df['higher_high'] = df['2. high'].rolling(window=2).max().shift(-1)
    return df

# Connect to Interactive Brokers
ib = IB()
ib.connect("127.0.0.1", 7497, clientId=1)

while True:
    most_traded_stocks = get_most_traded_stocks()

    for symbol in most_traded_stocks:
        stock_data = get_stock_data(symbol)

        if stock_data is not None:
            df_with_signals = higher_highs_strategy(stock_data)

            # Check for buy signal
            if df_with_signals.iloc[-1]["higher_high"]:
                contract = Stock(symbol, 'SMART', 'USD')
                ib.qualifyContracts(contract)
                order = Order(action="BUY", totalQuantity=100, orderType="MKT")
                trade = ib.placeOrder(contract, order)

                # Check for sell signal
                time.sleep(5)
                stock_data = get_stock_data(symbol)
                df_with_signals = higher_highs_strategy(stock_data)

                if not df_with_signals.iloc[-1]["higher_high"]:
                    order = Order(action="SELL", totalQuantity=100, orderType="MKT")
                    trade = ib.placeOrder(contract, order)

    time.sleep(5)
