import os
import csv
import pandas as pd
import requests
import time

api_key = "JACJKLH87O6AUMBM"

def get_stock_data(symbol, interval="5min", outputsize="full"):
    url = f"https://www.alphavantage.co/query?function=TIME_SERIES_INTRADAY&symbol={symbol}&interval={interval}&outputsize={outputsize}&apikey={api_key}"
    response = requests.get(url)

    if response.status_code != 200:
        raise ValueError("API request error: status code {}".format(response.status_code))
    else:
        data = response.json()
        if "Time Series ({})".format(interval) in data:
            data = data["Time Series ({})".format(interval)]
            df = pd.DataFrame.from_dict(data, orient="index")
            df.index = pd.to_datetime(df.index)
            df = df.astype(float)
            return df
        else:
            raise ValueError("API response error: data not found for symbol {}".format(symbol))

def read_tickers_from_csv(files):
    tickers = []
    for file in files:
        file_path = os.path.join('Kernel', 'Stocks', file)
        with open(file_path, newline='') as csvfile:
            reader = csv.reader(csvfile)
            for row in reader:
                tickers.append(row[0])
    return tickers

tickers = read_tickers_from_csv(['AMEX.csv','NASDAQ.csv','NYSE.csv'] )

end_date = pd.Timestamp.now().strftime("%Y-%m-%d")
start_date = (pd.Timestamp.now() - pd.Timedelta(days=90)).strftime("%Y-%m-%d")

for ticker in tickers:
    try:
        data = get_stock_data(ticker, interval="5min", outputsize="full")
        data = data.sort_index(ascending=True).loc[start_date:end_date]
        filename = "{}_5min.csv".format(ticker)  # Change to 5min
        data.to_csv(filename)
        print("Saved {} data to {}".format(ticker, filename))
    except ValueError as e:
        print("Error retrieving {} data: {}".format(ticker, str(e)))
    time.sleep(1)  # Add a sleep time to avoid hitting the API rate limit
