from alpha_vantage.timeseries import TimeSeries
from ib_insync import IB
from ib_insync import IB, Stock, MarketOrder
import threading
import time

class Algorithm:
    def __init__(self, alpha_vantage_api_key, ib_host='127.0.0.1', ib_port=7497):
        self.ts = TimeSeries(key=alpha_vantage_api_key, output_format='pandas')
        self.stocks = []
        self.strategy = 'Higher Highs'
        self.ib = IB()
        self.ib.connect(ib_host, ib_port, clientId=1)
        self.trading = False
        self.paused = False


    def set_strategy(self, strategy):
        self.strategy = strategy

    def add_stock(self, stock):
        self.stocks.append(stock)

    def start(self):
        self.trading = True
        self.paused = False
        trading_thread = threading.Thread(target=self.run_trading)
        trading_thread.start()

    def pause(self):
        self.paused = True

    def resume(self):
        self.paused = False

    def stop(self):
        self.trading = False

    def run_trading(self):
        while self.trading:
            if not self.paused:
                for stock in self.stocks:
                    # Fetch data from AlphaVantage
                    data, _ = self.ts.get_intraday(symbol=stock, interval='5min', outputsize='compact')
                    # Implement your trading strategy here
                    signal = self.evaluate_signal(stock, data)

                    if signal == 'BUY':
                        contract = Stock(stock, 'SMART', 'USD')
                        order = MarketOrder('BUY', 10)  # Replace 10 with the desired number of shares
                        trade = self.ib.placeOrder(contract, order)
                        self.ib.sleep(1)  # Give the order some time to be executed

            time.sleep(5)  # Pause for 5 seconds before checking again

    def evaluate_signal(self, stock, data):
        # Implement your trading strategy logic here
        # This is a placeholder that always returns 'BUY'
        return 'BUY'


